from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

T = TypeVar("T", bound="TopHistoryEntry")


@_attrs_define
class TopHistoryEntry:
    """A single entry in the top agent history.

    Attributes:
        agent_version_id (UUID): Agent version ID
        agent_name (str): Agent name
        miner_hotkey (str): Miner hotkey
        version_number (int): Version number
        final_score (float): Final score
        top_at (datetime.datetime): When this agent became top
        is_current_top (bool): Currently the top agent
        was_top (bool): Previously held top agent status
    """

    agent_version_id: UUID
    agent_name: str
    miner_hotkey: str
    version_number: int
    final_score: float
    top_at: datetime.datetime
    is_current_top: bool
    was_top: bool
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        agent_version_id = str(self.agent_version_id)

        agent_name = self.agent_name

        miner_hotkey = self.miner_hotkey

        version_number = self.version_number

        final_score = self.final_score

        top_at = self.top_at.isoformat()

        is_current_top = self.is_current_top

        was_top = self.was_top

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "agent_version_id": agent_version_id,
                "agent_name": agent_name,
                "miner_hotkey": miner_hotkey,
                "version_number": version_number,
                "final_score": final_score,
                "top_at": top_at,
                "is_current_top": is_current_top,
                "was_top": was_top,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        agent_version_id = UUID(d.pop("agent_version_id"))

        agent_name = d.pop("agent_name")

        miner_hotkey = d.pop("miner_hotkey")

        version_number = d.pop("version_number")

        final_score = d.pop("final_score")

        top_at = isoparse(d.pop("top_at"))

        is_current_top = d.pop("is_current_top")

        was_top = d.pop("was_top")

        top_history_entry = cls(
            agent_version_id=agent_version_id,
            agent_name=agent_name,
            miner_hotkey=miner_hotkey,
            version_number=version_number,
            final_score=final_score,
            top_at=top_at,
            is_current_top=is_current_top,
            was_top=was_top,
        )

        top_history_entry.additional_properties = d
        return top_history_entry

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.agent_latest_version import AgentLatestVersion


T = TypeVar("T", bound="AgentPublic")


@_attrs_define
class AgentPublic:
    """Public representation of an agent.

    Attributes:
        agent_id (UUID): Unique identifier for the agent
        miner_hotkey (str): Owner's hotkey
        agent_name (str): Human-readable name for the agent
        created_at (datetime.datetime): When the agent was created
        latest_version (AgentLatestVersion | None | Unset): Latest version with state and score
    """

    agent_id: UUID
    miner_hotkey: str
    agent_name: str
    created_at: datetime.datetime
    latest_version: AgentLatestVersion | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.agent_latest_version import AgentLatestVersion

        agent_id = str(self.agent_id)

        miner_hotkey = self.miner_hotkey

        agent_name = self.agent_name

        created_at = self.created_at.isoformat()

        latest_version: dict[str, Any] | None | Unset
        if isinstance(self.latest_version, Unset):
            latest_version = UNSET
        elif isinstance(self.latest_version, AgentLatestVersion):
            latest_version = self.latest_version.to_dict()
        else:
            latest_version = self.latest_version

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "agent_id": agent_id,
                "miner_hotkey": miner_hotkey,
                "agent_name": agent_name,
                "created_at": created_at,
            }
        )
        if latest_version is not UNSET:
            field_dict["latest_version"] = latest_version

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.agent_latest_version import AgentLatestVersion

        d = dict(src_dict)
        agent_id = UUID(d.pop("agent_id"))

        miner_hotkey = d.pop("miner_hotkey")

        agent_name = d.pop("agent_name")

        created_at = isoparse(d.pop("created_at"))

        def _parse_latest_version(data: object) -> AgentLatestVersion | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                latest_version_type_0 = AgentLatestVersion.from_dict(data)

                return latest_version_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(AgentLatestVersion | None | Unset, data)

        latest_version = _parse_latest_version(d.pop("latest_version", UNSET))

        agent_public = cls(
            agent_id=agent_id,
            miner_hotkey=miner_hotkey,
            agent_name=agent_name,
            created_at=created_at,
            latest_version=latest_version,
        )

        agent_public.additional_properties = d
        return agent_public

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.miner_agents_response import MinerAgentsResponse
from ...types import Response


def _get_kwargs() -> dict[str, Any]:

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/v1/miner/agents",
    }

    return _kwargs


def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> MinerAgentsResponse | None:
    if response.status_code == 200:
        response_200 = MinerAgentsResponse.from_dict(response.json())

        return response_200

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[MinerAgentsResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
) -> Response[MinerAgentsResponse]:
    """List miner's agents

     List all agents owned by the authenticated miner, with cooldown status.

    Includes the latest version per agent with its state and score so the
    frontend doesn't need to make N additional requests.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[MinerAgentsResponse]
    """

    kwargs = _get_kwargs()

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient | Client,
) -> MinerAgentsResponse | None:
    """List miner's agents

     List all agents owned by the authenticated miner, with cooldown status.

    Includes the latest version per agent with its state and score so the
    frontend doesn't need to make N additional requests.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        MinerAgentsResponse
    """

    return sync_detailed(
        client=client,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
) -> Response[MinerAgentsResponse]:
    """List miner's agents

     List all agents owned by the authenticated miner, with cooldown status.

    Includes the latest version per agent with its state and score so the
    frontend doesn't need to make N additional requests.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[MinerAgentsResponse]
    """

    kwargs = _get_kwargs()

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient | Client,
) -> MinerAgentsResponse | None:
    """List miner's agents

     List all agents owned by the authenticated miner, with cooldown status.

    Includes the latest version per agent with its state and score so the
    frontend doesn't need to make N additional requests.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        MinerAgentsResponse
    """

    return (
        await asyncio_detailed(
            client=client,
        )
    ).parsed
